export declare enum Type {
    CCML = 0,
    Collect = 1,
    Conference = 2,
    Debugger = 3,
    Detect = 4,
    Dial = 5,
    Function = 6,
    GoogleAnalytics = 7,
    Goto = 8,
    HangUp = 9,
    If = 10,
    Label = 11,
    Pause = 12,
    Play = 13,
    Queue = 14,
    Record = 15,
    Redirect = 16,
    Reject = 17,
    Say = 18,
    SessionData = 19,
    SMS = 20,
    Stop = 21,
    Transfer = 22,
    Var = 23,
    Webhook = 24,
}
export interface IAction {
    id: string;
    type: Type;
    params: any;
    events: any;
    name: string;
    comments: string;
    setParam(name: string, val: string): void;
    getParam(name: string): string;
    setEvent(name: string, val: IAction[]): void;
    getEvent(name: string): IAction[];
    toCCML(arg?: any): string;
}
export interface IActionParams {
    [index: string]: string;
}
export interface IActionEvents {
    [index: string]: IAction[];
}
export interface IActionData {
    id?: string;
    type: string;
    params?: any;
    events?: any;
    name?: string;
    comments?: string;
    actions?: IActionData[];
    session_data?: any;
    condition?: string;
    then?: any;
    orElse?: any;
}
export declare abstract class CoreAction implements IAction {
    private data;
    type: Type;
    params: IActionParams;
    events: IActionEvents;
    name: string;
    comments: string;
    static genId(prefix?: string): string;
    constructor(data: IActionData);
    readonly abstract id: string;
    setParam(name: string, val: string): void;
    getParam(name: string): string;
    setEvent(name: string, val: IAction[]): void;
    getEvent(name: string): IAction[];
    toCCML(paramMap?: any): string;
    escapeXml(unsafe: string): string;
    escapeDoubleQuotes(unsafe: string): string;
}
