import {
    IAction,
    IActionData,
    CoreAction,
    Type,
} from "./interfaces";
import {Say} from "./Say";
import {Play} from "./Play";


export class Collect extends CoreAction {

    private static _counter = 0;

    private _paramMap: any = {
        id            : "id",
        varName       : "varName",
        num_digits    : "maxKeys",
        finish_on_key : "endKey",
        digits_only   : "digitsOnly",
        enforce_end   : "enforceEndOnMax"
    };
    private _timeout = 5000;
    private _groupActions: IAction[] = [];


    constructor (public name: string = "action" + (++Collect._counter)) {

        super(<IActionData> {
            type: "COLLECT",
            name: name,
            comments: "",
            params: {
                id            : CoreAction.genId("collect"),
                varName       : "collect_keys_" + name,
                num_digits    : "1",
                finish_on_key : "pound",
                digits_only   : "false",
                enforce_end   : "true"
            }}
        );
        this.type = Type.Collect;
    }

    public get id (): string {
        return this.getParam("id");
    }

    public set numDigits (num: number) {
        this.setParam("num_digits", String(num));
    }
    public get numDigits (): number {
        return +this.getParam("num_digits");
    }

    public set finishOnKey (key: string) {
        this.setParam("finish_on_key", String(key));
    }
    public get finishOnKey (): string {
        return this.getParam("finish_on_key");
    }

    public set digitsOnly (key: boolean) {
        this.setParam("digits_only", String(key));
    }
    public get digitsOnly (): boolean {
        return this.getParam("digits_only") === "true";
    }

    public set timeout (ms: number) {
        this._timeout = +ms;
    }
    public get timeout (): number {
        return +this._timeout;
    }

    /**
     * Groups actions.
     * @param {IAction[]} actions - PLAY or SAY actions.
     */
    public set actions (actions: IAction[]) {
        this._groupActions = actions
            .filter(a => a.type === Type.Play || a.type === Type.Say || a.type === Type.Debugger)
            .map(a => {
                (<Play & Say>a).stopOnKey = true;
                return a;
            });
    }
    public get actions (): IAction[] {
        return this._groupActions;
    }

    public toCCML (): string {

        let id = CoreAction.genId();
        let ccml: string[] = [];

        // ccml.push(`<SET varName="___collectAttempt_${id}___" evaluate="true" export="false" value="(function () { return 1 + (\${___collectAttempt_${id}___} | 0); }());"/>`);
        ccml.push(`<SET varName="___collectAttempt_${id}___" evaluate="true" export="false" value="1 + (\${___collectAttempt_${id}___} | 0)"/>`);
        ccml.push(`<SET varName="___localVar_collectAttempt___" export="false" value="\${___collectAttempt_${id}___}"/>`);

        if (this._groupActions.length > 0) {
            this._groupActions.forEach(a => ccml.push(a.toCCML("sets")));
            ccml.push(`<GROUP repeatMax="0">`);
            this._groupActions.filter(a => a.type !== Type.Debugger).forEach(a => ccml.push(a.toCCML("says")));
            ccml.push(`</GROUP>`);
        }

        ccml.push(`<COLLECT${super.toCCML(this._paramMap)}>`);
        ccml.push(`<TIMER milliSeconds="${this._timeout}" nextActionId="\${next}"/>`);
        ccml.push(`</COLLECT>`);
        ccml.push(`<SET varName="___lastCollectKeys___" evaluate="true" value="\${${this.getParam("varName")}} || ''" export="false"/>`);
        ccml.push(`<SET varName="collect_keys" value="\${collect_keys}${this.name}=\${${this.getParam("varName")}}&amp;" export="true"/>`);

        return ccml.join("");
    }
}
