import {
    IAction,
    IActionData,
    CoreAction,
    Type,
} from "./interfaces";

import { VarParser } from "../utils";
import { If }        from "./If";
import { Say }       from "./Say";
import { Var }       from "./Var";

/**
 * Class for Debugger action.
 */
export class Debugger extends CoreAction {

    private _localVars: { [key: string]: string } = {};
    private _jsonActions: any;

    constructor(public name = "debugger") {

        super(<IActionData> {
            type: "DEBUGGER",
            name: name,
            comments: "",
            params: {},
        });
        this.type = Type.Debugger;
    }

    public get id (): string {
        return this.getParam("id");
    }

    public set jsonActions (js: any) {
        this._jsonActions = js;
    }

    public toCCML (nextAction: IAction): string {

        // If Var Action, log it and move on.
        if (nextAction instanceof Var) {
            this._localVars[nextAction.assignmentVar] = VarParser.parseExpression(nextAction.expression);
            return "";
        }

        // If first request of callback, export JSON Actions and Callback Config.
        const jsonActionsData = this._jsonActions ?
            `
                <SET varName="debugger_json_actions" evaluate="false" export="false"><![CDATA[ ${JSON.stringify(this._jsonActions)} ]]></SET>
                <SET varName="debugger_callback_config" value="\${___config___}" export="false"/>
            ` : "";
        this._jsonActions = undefined;

        // Export all local vars.
        const localVarObj = "{" + Object.keys(this._localVars).reduce((p, c) => p + `"${c}":${this._localVars[c]},`, "").slice(0, -1) + "}";
        const localVarData = `<SET varName="debugger_local_var_data" evaluate="true" export="false"><![CDATA[ JSON.stringify(${localVarObj}) ]]></SET>`;

        // If next Action is IF, export condition value.
        const ifConditionValue = (function () {
            if (nextAction instanceof If) {
                const ifExpr = VarParser.parseExpression(nextAction.condition);
                return `<SET varName="debugger_if_condition_eval" evaluate="true" export="false"><![CDATA[ JSON.stringify(${ifExpr}) ]]></SET>`;
            }
            return "";
        }());

        const sayTextValue = (function () {
            if (nextAction instanceof Say) {
                const resp = VarParser.parseVars(nextAction.ttsText);
                const text = resp[0];
                const varSets = resp[1].join("");
                return `${varSets}<SET varName="debugger_say_text_eval" evaluate="false" export="false"><![CDATA[ ${text.trim()} ]]></SET>`;
            }
            return "";
        })();

        return `
            ${sayTextValue} ${ifConditionValue} ${localVarData} ${jsonActionsData}
            <SET varName="debugger_action_id" value="${nextAction.id}" export="false"/>
            <WAIT nextActionId="TRIGGER_DEBUGGER"/>
        `;
    }
}
