import {
    IActionData,
    CoreAction,
    Type,
} from "./interfaces";
import { VarParser } from "../utils";

export const DialOnEvent: any = {
    OnError: "on_error",
};


export class Dial extends CoreAction {

    private _paramMap: any = {
        id: "id",
        caller_id_no: "from",
        detection: "detection",
        inbound_api: "campaignName"
    };

    private _sessionData = "";
    private _sessionDataType = "none";

    private _disableRedialBlock = false;

    constructor(private no = "", public name = "action") {

        super(<IActionData> {
            type: "DIAL",
            name: name,
            comments: "",
            params: {
                id: CoreAction.genId("dial"),
                caller_id_no: "${apiNo}",
                detection: "none",
                inbound_api: "__CallFlows__"
            },
            events: {
                [DialOnEvent.OnError]: [],
            }
        });
        this.type = Type.Dial;
    }

    public get id (): string {
        return this.getParam("id");
    }

    public set callerIdNo (from: string) {
        this.setParam("caller_id_no", String(from));
    }
    public get callerIdNo (): string {
        return this.getParam("caller_id_no");
    }

    public set detection (type: string) {
        this.setParam("detection", String(type));
    }
    public get detection (): string {
        return this.getParam("detection");
    }

    public set inboundAPI (api: string) {
        this.setParam("inbound_api", "__" + String(api) + "__");
    }
    public get inboundAPI (): string {
        let name = this.getParam("inbound_api");
        return name.substr(2, name.length - 2);
    }

    public set sessionData (data: any) {
        this._sessionDataType = typeof data;
        this._sessionData = JSON.stringify(data);
    }
    public get sessionData (): any {
        return JSON.parse(this._sessionData);
    }

    public set disableRedialBlock (v: boolean) {
        this._disableRedialBlock = v;
    }
    public get disableRedialBlock (): boolean {
        return this._disableRedialBlock;
    }

    public toCCML (): string {

        const debugModeSet = `<SET varName="___debugMode___" value="\${___isDebugMode___}"/>`;

        const sessionDataSet = (function (type: string, data: string) {

            if (type === "none") {
                return "";
            }
            if (type !== "string") {
                return `<SET varName="___sessionData___"><![CDATA[ ${data} ]]></SET>`;
            }
            try {
                const exp = VarParser.parseExpression(JSON.parse(data));
                return `<SET varName="___sessionData___" evaluate="true"><![CDATA[ JSON.stringify( ${exp} ) ]]></SET>`;
            } catch (err) {
                return `<!-- SessionData Error: ${err.message} -->`; // `<SET varName="___sessionData___"><![CDATA[ ${data} ]]></SET>`;
            }

        }(this._sessionDataType, this._sessionData));

        let ccml = "";

        const noVars = VarParser.parseVars(this.no);
        this.no = noVars[0];
        ccml += noVars[1].join("");

        const callerIdNoVars = VarParser.parseVars(this.callerIdNo, noVars[1].length);
        this.callerIdNo = callerIdNoVars[0];
        ccml += callerIdNoVars[1].join("");

        const dialOriginVars = `<SET varName="___dialOrigin_no___" export="false" value="${this.escapeXml(noVars[0])}"/>`;
        ccml += this._disableRedialBlock ? "" : `
            <SET varName="___dialOrigin_temp_no___" export="false" value="${this.escapeXml(noVars[0])}"/>
            <CONDITION>
                <WAIT condition="\${phoneRecordData.___dialOrigin_no___} == \${___dialOrigin_temp_no___} || \${___dialOrigin_no___} == \${___dialOrigin_temp_no___}">
                    <SAY>Error. A possible dial action recursive loop has been detected. Set disable redial block on the dial action to remove this check. Ending the call for your protection.</SAY>
                    <HANGUP/>
                </WAIT>
                <WAIT condition="true" nextActionId="\${next}"/>
            </CONDITION>
        `;

        if (this.events[DialOnEvent.OnError].length > 0) {
            const id = CoreAction.genId();
            if (sessionDataSet === "") {
                return `${ccml}
                    <DIAL${super.toCCML(this._paramMap)} maxIdleTime="21600" maxCallTime="86400" errorActionId="${id}_onError">${this.escapeXml(this.no)} ${dialOriginVars} ${debugModeSet}</DIAL>
                    <WAIT condition="true" nextActionId="${id}_onSuccess"/>
                    <BOOKMARK id="${id}_onError"/>
                        ${this.events[DialOnEvent.OnError].map(e => e.toCCML()).join("")}
                    <BOOKMARK id="${id}_onSuccess"/>
                `;
            } else {

                return `${ccml}
                    <SET varName="___local_sessionData___" value="\${___sessionData___}"/>
                    <DIAL${super.toCCML(this._paramMap)} maxIdleTime="21600" maxCallTime="86400" errorActionId="${id}_onError">${this.escapeXml(this.no)} ${dialOriginVars} ${sessionDataSet} ${debugModeSet}</DIAL>
                    <WAIT condition="true" nextActionId="${id}_onSuccess"/>
                    <BOOKMARK id="${id}_onError"/>
                        <SET varName="___sessionData___" value="\${___local_sessionData___}"/>
                        <SET varName="___local_sessionData___" value="\${NULL}"/>
                        ${this.events[DialOnEvent.OnError].map(e => e.toCCML()).join("")}
                        <WAIT condition="true" nextActionId="${id}_onSuccessSkipSessionData"/>
                    <BOOKMARK id="${id}_onSuccess"/>
                    <SET varName="___sessionData___" value="\${___local_sessionData___}"/>
                    <SET varName="___local_sessionData___" value="\${NULL}"/>
                    <BOOKMARK id="${id}_onSuccessSkipSessionData"/>
                `;
            }
        }

        if (sessionDataSet === "") {
            return `${ccml}
                <DIAL${super.toCCML(this._paramMap)} maxIdleTime="21600" maxCallTime="86400">${this.escapeXml(this.no)} ${dialOriginVars} ${debugModeSet}</DIAL>
            `;
        } else {
            return `${ccml}
                <SET varName="___local_sessionData___" value="\${___sessionData___}"/>
                <DIAL${super.toCCML(this._paramMap)} maxIdleTime="21600" maxCallTime="86400">${this.escapeXml(this.no)} ${dialOriginVars} ${sessionDataSet} ${debugModeSet}</DIAL>
                <SET varName="___sessionData___" value="\${___local_sessionData___}"/>
                <SET varName="___local_sessionData___" value="\${NULL}"/>
            `;
        }
    }
}
