import {
    IActionData,
    CoreAction,
    Type,
} from "./interfaces";
import { VarParser } from "../utils";


export class Goto extends CoreAction {

    private _paramMap: any = {
        id: "id",
        condition: "condition",
        nextActionId: "nextActionId"
    };

    constructor(label: string, public name = "action") {

        super(<IActionData> {
            type: "GOTO",
            name: name,
            comments: "",
            params: {
                id: CoreAction.genId("goto"),
                condition: "true",
                nextActionId: `__labelId_${label}__` // See LABEL id.
            }
        });
        this.type = Type.Goto;
    }

    public get id (): string {
        return this.getParam("id");
    }

    public get label () {
        return this.getParam("nextActionId").slice(10, -2);
    }

    public toCCML(): string {

        let noVars = VarParser.parseVars(this.getParam("nextActionId"));
        if (noVars[1].length > 0) {
            const ccml = noVars[1].concat(
                `<SET varName="___temp_gotoLabel___" value="${this.escapeXml(noVars[0])}" export="false"/>`
            ).join("");
            this.setParam("nextActionId", "${___temp_gotoLabel___}"); // noVars[0]);
            return `${ccml}<WAIT${super.toCCML(this._paramMap)}/>`;
        }

        return `<WAIT${super.toCCML(this._paramMap)}/>`;
    }
}
