import {
    IAction,
    IActionData,
    CoreAction,
    Type,
} from "./interfaces";
import { VarParser } from "../utils";

export const IfEvent = {
    OnThen: "on_then",
    OnElse: "on_else",
};

/**
 {
     "type": "IF",
    "condition": "$sessionData.name === 'Bob'",
    "then": [],
    "else": []
    }
    */
export class If extends CoreAction {

    private _paramMap: any = {
        id: "id"
    };

    constructor(private _condition = "true", then: IAction[] = [], public name = "action") {

        super(<IActionData> {
            type: "IF",
            name: name,
            comments: "",
            params: {
                id: CoreAction.genId("if")
            },
            events: {
                on_then: then,
                on_else: []
            }
        });
        this.type = Type.If;
    }

    public get id (): string {
        return this.getParam("id");
    }

    public get condition () {
        return this._condition;
    }

    public get then () {
        return this.getEvent(IfEvent.OnThen);
    }

    public get orElse () {
        return this.getEvent(IfEvent.OnElse);
    }

    public toCCML (): string {

        this._condition = VarParser.parseExpression(this._condition);

        let id = CoreAction.genId();
        let paramStr: string = super.toCCML(this._paramMap);
        let ccml = `
            <CONDITION${paramStr}>
                <WAIT nextActionId="${id}_ifThen"><![CDATA[${this._condition}]]></WAIT>
                <WAIT condition="true" nextActionId="${id}_orElse"/>
            </CONDITION>
            <BOOKMARK id="${id}_ifThen"/>
                ${this.getEvent(IfEvent.OnThen).map(a => a.toCCML()).join("")}
                <WAIT condition="true" nextActionId="${id}_endIf"/>
            <BOOKMARK id="${id}_orElse"/>
                ${this.getEvent(IfEvent.OnElse).map(a => a.toCCML()).join("")}
            <BOOKMARK id="${id}_endIf"/>
        `;

        return ccml;
    }
}
