import {
    IActionData,
    CoreAction,
    Type,
} from "./interfaces";
import { VarParser } from "../utils";

export class Play extends CoreAction {

    private _paramMap: any = {
        id: "id",
        url: "url",
        stop_on_key: "stopOnKeyPressed",
        dtmf: "dtmf",
        dtmf_delay: "digitDelay",
        dtmf_dash_delay: "pauseDelay",
        repeatMax: "repeatMax"
    };
    private _repeat = 1;
    private _parseVars = true;
    private _injectCCML = "";

    /**
     * Constructor.
     * @param url - url or dtmf
     * @param name
     */
    constructor (url = "", public name = "action") {

        super(<IActionData> {
            type: "PLAY",
            name: name,
            comments: "",
            params: url.indexOf("http") === 0 ? {
                id: CoreAction.genId("play"),
                url: url,
                stop_on_key: "false",
                repeatMax: "0"
            } : {
                id: CoreAction.genId("dtmf"),
                dtmf: url,
                dtmf_delay: 250,
                dtmf_dash_delay: 250,
            }
        });
        this.type = Type.Play;
    }

    get id (): string {
        return this.getParam("id");
    }

    set repeat (num: number) {
        this._repeat = Math.min(Math.max(1, +num), 10);
    }
    get repeat (): number {
        return this._repeat;
    }

    set parseVars (val: boolean) {
        this._parseVars = val;
    }
    get parseVars (): boolean {
        return this._parseVars;
    }

    set stopOnKey (stop: boolean) {
        this.setParam("stop_on_key", stop ? "true" : "false");
    }
    get stopOnKey (): boolean {
        return this.getParam("stop_on_key") === "true";
    }

    set dtmfDelay (v: number) {
        this.setParam("dtmf_delay", String(v));
    }
    get dtmfDelay () {
        return +this.getParam("dtmf_delay");
    }

    set dtmfDashDelay (v: number) {
        this.setParam("dtmf_dash_delay", String(v));
    }
    get dtmfDashDelay () {
        return +this.getParam("dtmf_dash_delay");
    }

    set injectCCML (c: string) {
        this._injectCCML = c || "";
    }
    get injectCCML () {
        return this._injectCCML;
    }

    public toCCML (filter?: "sets" | "says"): string {

        return this.getParam("url") ? this.playToCCML(filter) : this.dtmfToCCML(filter);
    }

    private playToCCML (filter?: "sets" | "says"): string {

        let ccml: string[] = [];

        if (this._parseVars && filter !== "says") {
            let resp = VarParser.parseVars(this.getParam("url"));
            this.setParam("url", resp[0]);
            ccml = ccml.concat(resp[1]);
        }

        if (filter !== "sets") {
            if (this.repeat > 1) {

                delete this._paramMap.id;

                let action = `<PLAY${super.toCCML(this._paramMap)} ${this.injectCCML ? "/>" + this.injectCCML + "<PLAY" : ""}/>`;
                for (let i = 0; i < this.repeat; i++) {
                    ccml.push(action);
                }

            } else {
                ccml.push(`<PLAY${super.toCCML(this._paramMap)} ${this.injectCCML ? "/>" + this.injectCCML + "<PLAY" : ""}/>`);
            }
        }

        return ccml.join("");
    }

    private dtmfToCCML (filter?: "sets" | "says"): string {

        let ccml: string[] = [];

        this.setParam("dtmf", this.getParam("dtmf").replace(/-/g, ","));

        if (this._parseVars && filter !== "says") {
            let resp = VarParser.parseVars(this.getParam("dtmf"));
            this.setParam("dtmf", resp[0]);
            ccml = ccml.concat(resp[1]);
        }

        if (filter !== "sets") {
            if (this.repeat > 1) {

                delete this._paramMap.id;

                let action = `<SENDDTMF${super.toCCML(this._paramMap)}/>`;
                for (let i = 0; i < this.repeat; i++) {
                    ccml.push(action);
                }

            } else {
                ccml.push(`<SENDDTMF${super.toCCML(this._paramMap)}/>`);
            }
        }

        return ccml.join("");
    }
}
