import {
    IActionData,
    CoreAction,
    Type,
} from "./interfaces";
import {VarParser} from "../utils";


export class Redirect extends CoreAction {

    // private _paramMap: any = {};

    private _callbackMethod: string;
    private _callbackHeaders: any;
    private _callbackContentType: string;

    constructor(private _callbackUrl = "", public name = "action") {

        super(<IActionData> {
            type: "REDIRECT",
            name: name,
            comments: "",
            params: {
                id: CoreAction.genId("redirect"),
            }
        });
        this.type = Type.Redirect;
    }

    public get id (): string {
        return this.getParam("id");
    }

    public get url () {
        return this._callbackUrl;
    }

    public set method (val: string) {
        this._callbackMethod = val;
    }
    public get method () {
        return this._callbackMethod;
    }

    public set headers (val: any) {
        this._callbackHeaders = val;
    }
    public get headers () {
        return this._callbackHeaders;
    }

    public set contentType (val: string) {
        this._callbackContentType = val;
    }
    public get contentType () {
        return this._callbackContentType;
    }

    public toCCML(): string {

        const ccml: string[] = [];

        if (this._callbackUrl) {
            const [url, varSetup] = VarParser.parseVars(this._callbackUrl);
            ccml.push(...varSetup);
            ccml.push(`
                <SET varName="___redirectTempUrl___" value=${JSON.stringify(url)} export="false"/>
                <SET varName="___dynamicCallbackUrl___" evaluate="true" export="true"><![CDATA[
                    (function (orig, rel) {
                        if (rel.indexOf('://') !== -1) return rel;
                        relParts = rel.split("../");
                        origParts = orig.split("/").slice(0, -relParts.length);
                        return [].concat(origParts, relParts.pop()).join("/");
                    }(\${___dynamicCallbackUrl___}, \${___redirectTempUrl___}))
                ]]></SET>
            `);
        }

        if (this._callbackMethod) {
            const [method, varSetup] = VarParser.parseVars(this._callbackMethod);
            ccml.push(...varSetup);
            ccml.push(`<SET varName="___dynamicCallbackMethod___" value=${JSON.stringify(method)} export="true"/>`);
        }

        if (this._callbackHeaders) {
            ccml.push(`<SET varName="___dynamicCallbackHeaders___" value=${JSON.stringify(JSON.stringify(this._callbackHeaders))} export="true"/>`);
        }

        if (this._callbackContentType) {
            const [contentType, varSetup] = VarParser.parseVars(this._callbackContentType);
            ccml.push(...varSetup);
            ccml.push(`<SET varName="___dynamicCallbackContentType___" value=${JSON.stringify(contentType)} export="true"/>`);
        }

        return ccml.join("");
    }
}
