import {
    IActionData,
    CoreAction,
    Type,
} from "./interfaces";
import {VarParser} from "../utils";


export class Say extends CoreAction {

    private _paramMap: any = {
        id: "id",
        voice: "voice",
        language: "language",
        gender: "gender",
        stop_on_key : "stopOnKeyPressed",
        repeatMax: "repeatMax"
    };
    private _repeat = 1;
    private _parseVars = true;
    private _injectCCML = "";

    constructor(private text = "This say action text is missing.", public name = "action") {

        super(<IActionData> {
            type: "SAY",
            name: name,
            comments: "",
            params: {
                id : CoreAction.genId("say"),
                stop_on_key: "false",
                voice: "default",
                language: "default",
                gender: "default",
                repeatMax: "0"
            },
        });
        this.type = Type.Say;
    }

    public get id(): string {
        return this.getParam("id");
    }

    public get ttsText (): string {
        return this.text;
    }

    public set repeat(num: number) {
        this._repeat = Math.min(Math.max(1, +num), 10);
    }
    public get repeat(): number {
        return this._repeat;
    }

    public set parseVars (val: boolean) {
        this._parseVars = val;
    }
    public get parseVars(): boolean {
        return this._parseVars;
    }

    public set stopOnKey (val: boolean) {
        this.setParam("stop_on_key", val ? "true" : "false");
    }
    public get stopOnKey (): boolean {
        return this.getParam("stop_on_key") === "true";
    }

    public set voice (val: string) {
        this.setParam("voice", val);
    }
    public get voice (): string {
        return this.getParam("voice");
    }

    public set language (val: string) {
        this.setParam("language", val);
    }
    public get language (): string {
        return this.getParam("language");
    }

    public set gender (val: string) {
        this.setParam("gender", val);
    }
    public get gender (): string {
        return this.getParam("gender");
    }

    public set injectCCML (c: string) {
        this._injectCCML = c || "";
    }
    public get injectCCML () {
        return this._injectCCML;
    }

    public toCCML(filter?: "sets" | "says"): string {

        let ccml: string[] = [];

        if (filter !== "says") {
            if (this._parseVars) {
                let resp = VarParser.parseVars(this.text);
                this.text = resp[0];
                ccml = ccml.concat(resp[1]);
            }
        }

        if (filter !== "sets") {
            let txt = "";
            for (let i = 0; i < this.repeat; i++) {
                txt += this.text + " ";
            }
            ccml.push(`<SAY${super.toCCML(this._paramMap)}><![CDATA[ ${txt.trim()} ]]>${this.injectCCML}</SAY>`);
        }

        return ccml.join("");
    }
}
