import {
    IActionData,
    CoreAction,
    Type,
} from "./interfaces";
import { VarParser } from "../utils";


export class Var extends CoreAction {

    // private _paramMap: any = {
    //     id: "id"
    // };

    constructor(_assignmentVar: string, _expression: string, public name: string = _assignmentVar) {

        super(<IActionData> {
            type: "VAR",
            name: name,
            comments: `${_assignmentVar} = ${_expression}`,
            params: {
                id: CoreAction.genId("var"),
                assignmentVar : _assignmentVar,
                expression: _expression,
            },
        });
        this.type = Type.Var;
    }

    public get id (): string {
        return this.getParam("id");
    }

    public get assignmentVar () {
        return this.getParam("assignmentVar");
    }

    public get expression () {
        return this.getParam("expression");
    }

    public toCCML (): string {

        const av = VarParser.parseAssignmentVar(this.assignmentVar);
        if (av === undefined) {
            return "";
        }
        const exp = VarParser.parseExpression(this.expression);

        if (! av[1]) {
            return `
                <SET varName="${av[0]}" evaluate="true" export="false"><![CDATA[ JSON.stringify(${exp}) ]]></SET>
            `;
        } else {
            return `
                <SET varName="${av[2]}" evaluate="true" export="false"><![CDATA[ JSON.stringify(${exp}) ]]></SET>
                <SET varName="${av[0]}" evaluate="true" export="false"><![CDATA[
                    (function () {
                        try {
                            var data = JSON.parse(\${${av[0]}});
                            data${av[1]} = JSON.parse(\${${av[2]}});
                            return JSON.stringify(data);
                        } catch (e) {
                            return \${${av[0]}};
                        }
                    }())
                ]]></SET>
            `;
        }
    }
}
