
export enum Type {
    CCML, Collect, Conference, Debugger, Detect, Dial, Function, GoogleAnalytics, Goto, HangUp, If, Label, Pause,
    Play, Queue, Record, Redirect, Reject, Say, SessionData, SMS, Stop, Transfer, Var, Webhook
}

export interface IAction {

    id: string;
    type: Type;
    params: any;
    events: any;
    name: string;
    comments: string;

    setParam (name: string, val: string): void;
    getParam (name: string): string;

    setEvent (name: string, val: IAction[]): void;
    getEvent (name: string): IAction[];

    toCCML (arg?: any): string;
}

export interface IActionParams {
    [index: string]: string;
}

export interface IActionEvents {
    [index: string]: IAction[];
}

export interface IActionData {
    id?: string;
    type: string;
    params?: any;
    events?: any;
    name?: string;
    comments?: string;
    actions?: IActionData[];
    session_data?: any;
    condition?: string;
    then?: any;
    orElse?: any;
}

let idCounter = 10000;

export abstract class CoreAction implements IAction {

    public type: Type;
    public params: IActionParams;
    public events: IActionEvents;
    public name: string;
    public comments: string;


    public static genId (prefix = "") {
        if (prefix === "") {
            prefix += "_";
        }
        idCounter++;
        return prefix + idCounter.toString(36); // Date.now().toString(36) + Math.random().toString(36).substr(2);
    }


    constructor (private data: IActionData) {
        this.params = this.data.params || {};
        this.events = this.data.events || {};
        this.name = this.data.name || "";
        this.comments = this.data.comments || "";
    }

    abstract get id (): string;

    public setParam (name: string, val: string): void {
        this.params[name] = val;
    }

    public getParam (name: string): string {
        return this.params[name];
    }

    public setEvent (name: string, val: IAction[]): void {
        this.events[name] = val;
    }

    public getEvent (name: string): IAction[] {
        return this.events[name] || [];
    }

    public toCCML (paramMap?: any): string {

        return Object.keys(paramMap)
            .filter(k => this.params[k] !== null && this.params[k] !== undefined && this.params[k] !== "")
            .reduce((p, k) => `${p} ${paramMap[k]}=${JSON.stringify(String(this.params[k]))}`, "");
    }

    public escapeXml (unsafe: string) {
        return unsafe.replace(/[<>&'"]/g, function (c) {
            switch (c) {
                case "<" : return "&lt;";
                case ">" : return "&gt;";
                case "&" : return "&amp;";
                case "\'": return "&apos;";
                case "\"": return "&quot;";
                default  : return c;
            }
        });
    }

    public escapeDoubleQuotes (unsafe: string) {
        return unsafe.replace(/["]/g, "\\\"");
    }
}
