
import { IActionFunctionData } from "./interfaces";
import { CCMLBuilder }         from "../CCMLBuilder";
import { IAction }             from "../actions";
import { VarParser }           from "../utils";


export class ActionFunction {

    private _id: string;
    private _name: string;
    private _args: { [key: string]: string };
    private _actions: Array<IAction>;
    private _returns: { [key: string]: string };


    constructor (data: IActionFunctionData) {

        this._id = Math.floor(Math.random() * Number.MAX_SAFE_INTEGER).toString(36);
        this._name = data.name;
        this._args = data.args || {}; // Object.assign({}, data.args);
        this._actions = new CCMLBuilder(data).getActions();
        this._returns = data.returns || {}; // Object.assign({}, data.returns);
    }

    public get id () { return this._id; }
    public get name () { return this._name; }
    public get args () { return this._args; } // Object.assign({}, this._args); }
    public get actions () { return this._actions.slice(0); }
    public get returns () { return this._returns; } // Object.assign({}, this._returns); }


    public toCCML () {

        const actionString = this._actions.map(a => a.toCCML().replace(/___localVar_/g, `___funcVar_${this._name}_`)).join("");
        const returnMap = Object.keys(this._returns).reduce((p, k) => {
            const exp = VarParser.parseExpression(this._returns[k]).replace(/___localVar_/g, `___funcVar_${this._name}_`);
            p += `, ${JSON.stringify(k)}: ${exp}`;
            return p;
        }, `{ "_id": ${JSON.stringify(this._id)}, "_name": ${JSON.stringify(this._name)} `) + " }";

        return `
            <!-- Function ${this._name} -->
            <SUBSCRIPT id="___func_${this._name}___">
                ${actionString}
                <SET varName="___actionFunctionData___" evaluate="true" export="false"><![CDATA[ (function(){ return JSON.stringify(${returnMap}); }()) ]]></SET>
            </SUBSCRIPT>
        `;
    }
}


export function buildArgSets(name: string, args: { [key: string]: string }): string {

    return Object.keys(args).reduce((p, k) => {
        const exp = VarParser.parseExpression(args[k]);
        return p += `<SET varName="___funcVar_${name}_${k.replace(/^\$/, "")}___" evaluate="true" export="false"><![CDATA[ (function () { return JSON.stringify(${exp}); }()) ]]></SET>`;
    }, "");
}
