
const OBJ_MAP: any = {
    sessionData: "${___sessionData___}",
    webhookData: "${___webhookData___}",
    functionData: "${___actionFunctionData___}",
    config: "${___config___}",
    lastQueuePollConnectCall: "${___lastQueuePollConnectCall___}",
};
const OBJ_MAP_KEYS = Object.keys(OBJ_MAP);
const VAR_MAP: any = {
    isDebugMode: "${___isDebugMode___}",
    isInitCall: "${___isInit___}",
    now: "${timestamp}",
    callTime: "${callTimeMS}",
    direction: "${callType}",   // INBOUND or OUTBOUND
    startTime: "${___startTime___}",
    callerNo: "$PN+{phoneNoFarEndEx.number}", // OLD: phonrNo
    "...callerNo": "$PN{phoneNoFarEndEx.number}",
    callerNoRegion: "${phoneNoFarEndEx.state}",
    callerNoCountry: "${phoneNoFarEndEx.country}",
    callerNoTimeZone: "${phoneNoFarEndEx.timeZone}",
    apiNo: "$PN+{phoneNoNearEndEx.number}",    // OLD: from
    "...apiNo": "$PN{phoneNoNearEndEx.number}",
    apiNoRegion: "${phoneNoNearEndEx.state}",
    apiNoCountry: "${phoneNoNearEndEx.country}",
    apiNoTimeZone: "${phoneNoNearEndEx.timeZone}",
    callId: "${callId}",
    appId: "${app_id}",
    keyPresses: "${___lastCollectKeys___}", // From COLLECT action.
    numConferenceParticipants: "${conference.participants}",
    "...numConferenceParticipants": "$N{conference.participants}",
    // Below are deprecated vars.
    callerNoSay: "$PN{phoneNo}",
    "...callerNoSay": "$PN{phoneNo}",
    apiNoSay: "$PN{from}",
    "...apiNoSay": "$PN{from}",
    keyPressesSay: "$N{___lastCollectKeys___}", // From COLLECT action.
    "...keyPressesSay": "$N{___lastCollectKeys___}", // From COLLECT action.
};
const VAR_MAP_KEYS = Object.keys(VAR_MAP);

const RE_FIND_ALL_VARS         = /\${(\.+)?\w+([.\[][.a-zA-Z0-9\-_$'"\[\]]+)?}/g;
const RE_FIND_SESSION_DATA_VAR = /^\$(sessionData|webhookData|functionData)((?:.|\[)[a-zA-Z0-9_\-$'"\[\].]+)?$/;
const RE_FIND_LOCAL_VAR        = /^\$(\w+)((?:.|\[)[a-zA-Z0-9_\-$'"\[\].]+)?$/;
// const RE_FIND_ALL_LOCAL_VARS   = /([^a-zA-Z0-9_$.\]])\$(\w+)([^a-zA-Z0-9_$])/g;
// This one ignores ${VAR}, $N{VAR}, $PN{VAR} or $PN+{VAR}
const RE_FIND_ALL_LOCAL_VARS   = /([^a-zA-Z0-9_$.\]])\$(?!(?:[PN+]*\{))(\w+)([^a-zA-Z0-9_$\{])/g;
const RE_ESCAPE_RE_CHARS       = /[\-\[\]\/{}()*+?.\\\^$|]/g;

/**
 * Takes a string that supports template literals, generates SET tags and/or replaces the literals.
 * @param {string} str - string with template literals
 * @param {numner} [startIndex=0] - start index of ___temp_str_var_n___
 * @returns {[string,string[]]} - 1: the string replacement. 2: CCML array of ___temp_str_var_n___ SETs.
 */
export function parseVars (str: string, startIndex = 0): [string, string[]] {

    let ccml: string[] = [],
        id = "___temp_str_var";

    str = ` ${str} `; // Helps regex expression.

    if (RE_FIND_ALL_VARS.test(str)) {

        (str.match(RE_FIND_ALL_VARS) || [])
           .map(m => m.slice(2, -1))
           .filter(m => ! VAR_MAP[m])
           .forEach(m => {
                // Find if var prefixed with (.+)
                const [, splits, obj] = m.match(/^(\.*)(.+)/) || ["", "", ""];
                const splitsSuffix = splits ? `.toString().split("").join(",")` : "";
                const expression = parseExpression("$" + obj);
                const varName = `${id}_${startIndex++}___`;
                if (splitsSuffix) {
                    str = str.replace(new RegExp("\\${(\\.+)" + escapeRegExp(obj) + "}", "g"), "${" + varName + "}");
                } else {
                    str = str.replace(new RegExp("\\${" + escapeRegExp(obj) + "}", "g"), "${" + varName + "}");
                }
                ccml.push(`<SET varName="${varName}" export="false" evaluate="true"><![CDATA[ ${expression}${splitsSuffix} ]]></SET>`);
            });
    }

    str = VAR_MAP_KEYS.reduce((p, c) => p.replace(new RegExp(`\\\${${c}}`, "g"), VAR_MAP[c]), str);

    return [str.trim(), ccml];
}

export function parseAssignmentVar (v: string): [string, string, string] | undefined {

    let match = v.trim().match(RE_FIND_SESSION_DATA_VAR);
    if (match !== null) {
        // return ["___sessionData___", match[2], "___var_assign_temp___"];
        return [`___${match[1]}___`, match[2], "___var_assign_temp___"];
    }

    match = v.trim().match(RE_FIND_LOCAL_VAR);
    if (match === null) {
        return;
    }
    // ['___localVar_foo12345678___', '.bar12', '___var_assign_temp___']
    return [`___localVar_${match[1]}___`, match[2], "___var_assign_temp___"];
}

export function parseExpression (exp: string): string {

    exp = ` ${exp} `; // Helps regex expression.

    // Find/replace JS objects sessionData, config
    exp = OBJ_MAP_KEYS.reduce((p, c) => {

        // First regexp checks for objects with accessors ($config.foo). Second checks for plain objects ($config).
        [
            new RegExp(`[^a-zA-Z0-9_\\$](\\$` + c + `[\\.\\[][\\.a-zA-Z0-9_\\-\\$'"\\[\\]]+)[^a-zA-Z0-9_\\$]`, "g"),
            new RegExp(`[^a-zA-Z0-9_\\$](\\$` + c + `)[^a-zA-Z0-9_\\$]`, "g"),
        ].forEach(re => {
            let match = re.exec(p);
            while (match !== null) {
                let m = match[1].replace("$" + c, "JSON.parse(" + OBJ_MAP[c] + ")");
                p = p.replace(match[1], m);
                match = re.exec(p);
            }
        });
        return p;

    }, exp);

    // Find/replace JS vars
    exp = VAR_MAP_KEYS.reduce(
        (p, c) => p.replace(new RegExp(`[^a-zA-Z0-9_\\$](\\$${c})[^a-zA-Z0-9_\\$]`, "g"), ` ${VAR_MAP[c]} `),
        exp,
    );

    // Find/replace $foo to  ${___localVars___}.foo
    exp = exp.replace(RE_FIND_ALL_LOCAL_VARS, "$1(JSON.parse(${___localVar_$2___}))$3");

    return exp.trim();
}

function escapeRegExp (str: string) {
    return str.replace(RE_ESCAPE_RE_CHARS, "\\$&");
}
