"use strict";

var assert = require("assert");

var Utils = require('../lib/index.js').Utils;


describe('Parsing Vars', function () {

    it('should parse string into CCML vars', function () {

        var result = Utils.VarParser.parseVars('Var ${sessionData} here.');
        console.log(result);
        console.log();

        var result = Utils.VarParser.parseVars('Var ${webhookData} here.');
        console.log(result);
        console.log();

        result = Utils.VarParser.parseVars('Var ${sessionData.foo} ${sessionData.foo} here.');
        console.log(result);
        console.log();

        result = Utils.VarParser.parseVars('Var ${sessionData["bar"].foo} a ${sessionData["bar"].foo} here');
        console.log(result);
        console.log();

        result = Utils.VarParser.parseVars('Var ${sessionData["bar"].foo} a  ${sessionData.foo} a ${sessionData["bar"].foo} a ${sessionData.foo} here');
        console.log(result);
        console.log();

        result = Utils.VarParser.parseVars('Var ${foo} a ${foo} here ${now} or ${callTime}.');
        console.log(result);
        console.log();

        result = Utils.VarParser.parseVars('Var ${foo.bar} here');
        console.log(result);
        console.log();

        result = Utils.VarParser.parseVars('Var ${bar["foo"].bar} here');
        console.log(result);
        console.log();

        result = Utils.VarParser.parseVars('Var ${foo.bar} Var ${now} or ${callTime} a ${$bar["foo"].bar} Var ${sessionData} Var ${sessionData.foo} Var ${foo} here');
        console.log(result);
        console.log();
    });

    it('should parse assignment var $sessionData and local vars', function () {

        var result = Utils.VarParser.parseAssignmentVar('$sessionData');
        console.log(result);
        console.log();

        var result = Utils.VarParser.parseAssignmentVar('$webhookData');
        console.log(result);
        console.log();

        result = Utils.VarParser.parseAssignmentVar('$sessionData.foo');
        console.log(result);
        console.log();

        result = Utils.VarParser.parseAssignmentVar('$sessionData["bar"].foo');
        console.log(result);
        console.log();

        result = Utils.VarParser.parseAssignmentVar('$foo');
        console.log(result);
        console.log();

        result = Utils.VarParser.parseAssignmentVar('$foo.bar');
        console.log(result);
        console.log();

        result = Utils.VarParser.parseAssignmentVar('$bar["foo"].bar');
        console.log(result);
        console.log();
    });

    it('should parse expression $sessionData, built-in vars and local vars.', function () {

        var result = Utils.VarParser.parseExpression('3 + $sessionData.foo["foo"].bar * 3 - "boo" - $now');
        console.log(result);
        console.log();

        var result = Utils.VarParser.parseExpression('3 + $webhookData.foo["foo"].bar * 3 - "boo" - $now');
        console.log(result);
        console.log();

        result = Utils.VarParser.parseExpression('$foo * 3 + $sessionData.foo');
        console.log(result);
        console.log();

        result = Utils.VarParser.parseExpression('($iterator)[$index]');
        console.log(result);
        console.log();

        result = Utils.VarParser.parseExpression('$foo.bar - 3');
        console.log(result);
        console.log();

        result = Utils.VarParser.parseExpression('$bar["foo"].bar - 3');
        console.log(result);
        console.log();

        result = Utils.VarParser.parseExpression('$abc * 3 * $de1.bar - 3 * $bar["foo"].bar - 3');
        console.log(result);
        console.log();

    });

});
